{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Comparison Between Approximate P+M Solution and Strain Compatibility Analysis\n",
    "\n",
    "See blog post here: https://robwang.io/engineering/2021/10/22/wallflexure.html\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Import\n",
    "import math\n",
    "import numpy as np\n",
    "import matplotlib.pyplot as plt\n",
    "import matplotlib.patches as patches\n",
    "import time\n",
    "import itertools\n",
    "import os\n",
    "import ipywidgets"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Constants\n",
    "REBAR = {\n",
    "    3:[0.375, 0.11],\n",
    "    4:[0.500, 0.20],\n",
    "    5:[0.625, 0.31],\n",
    "    6:[0.750, 0.44],\n",
    "    7:[0.875, 0.60],\n",
    "    8:[1.000, 0.79],\n",
    "    9:[1.128, 1.00],\n",
    "    10:[1.270, 1.27],\n",
    "    11:[1.410, 1.56],\n",
    "    14:[1.693, 2.25],\n",
    "    18:[2.257, 4.00]}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Define Section class"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [],
   "source": [
    "class Section:\n",
    "    \"\"\"\n",
    "    Attributes:\n",
    "        f'c = concrete strength\n",
    "        fy = steel strength\n",
    "        b = wall thickness\n",
    "        h = wall length\n",
    "        cb = clear cover = 2 in\n",
    "        nx = number of column of bars in boundary element = 3\n",
    "        ny = number of rows of bars in boundary element\n",
    "        BE_bar = boundary element bar size\n",
    "        Lbe = boundary element length\n",
    "        web_bar = web bar size\n",
    "        s_target = target spacing in web\n",
    "        n_curtain = number of curtains in web = 2\n",
    "\n",
    "        P_max = maximum allowed axial load\n",
    "        ext_pt = list of points of four corners of wall\n",
    "        reinf_coord = rebar location coordinate\n",
    "        PMsurface = coordinate for P+M surface\n",
    "        PMsurface_approx = coordinate of P+M surface using approximate equation\n",
    "    \"\"\"\n",
    "    def __init__(self,fpc,fy,b,h,cb,nx,ny,BE_bar,Lbe,web_bar,s_target,n_curtain,zeta):\n",
    "        self.fpc = fpc\n",
    "        self.fy = fy\n",
    "        self.b = b\n",
    "        self.h = h\n",
    "        self.cb = cb\n",
    "        self.nx = nx\n",
    "        self.ny = ny\n",
    "        self.BE_bar = BE_bar\n",
    "        self.Lbe = Lbe\n",
    "        self.web_bar = web_bar\n",
    "        self.s_target = s_target\n",
    "        self.n_curtain = n_curtain\n",
    "        self.zeta = zeta\n",
    "        self.ext_pt = self.get_ext_pts()\n",
    "        self.BE_reinf,self.web_reinf,self.reinf_coord = self.get_reinf_coord()\n",
    "        self.determine_PM()\n",
    "        self.determine_approx_PM()\n",
    "        self.factored_PM()\n",
    "    \n",
    "\n",
    "    def get_ext_pts(self):\n",
    "        ext_pts=[[-self.b/2,self.b/2,self.b/2,-self.b/2,-self.b/2,],\n",
    "        [-self.h/2,-self.h/2,self.h/2,self.h/2,-self.h/2,]]\n",
    "        return ext_pts\n",
    "\n",
    "\n",
    "\n",
    "    def draw_BE_bars(self,x,y,b,h,cb,nx,ny,A,D,layout):\n",
    "        \"\"\"\n",
    "        This function generates the x,y coordinate of boundary element steel based on\n",
    "        the following input:\n",
    "        x : dx from origin to lower left of boundary element\n",
    "        y : dy from origin to lower left of boundary element\n",
    "        b : dimension of BE along x\n",
    "        h : dimension of BE along y\n",
    "        cb : clear cover to face of rebar\n",
    "        nx : number of columns along x\n",
    "        ny : number of columns along y\n",
    "        A : area of rebar\n",
    "        D : diameter of rebar\n",
    "        layout : layout. f for \"full\" or p for \"perimeter\"\n",
    "        \"\"\"\n",
    "        # first determine spacing and coordinate with local origin at lower left\n",
    "        cx = b - 2*cb - D\n",
    "        cy = h - 2*cb - D\n",
    "        sx = 0 if nx==1 else cx / (nx-1)\n",
    "        sy = 0 if ny==1 else cy / (ny-1)\n",
    "            \n",
    "        xcoord=[]\n",
    "        ycoord=[]\n",
    "        xcoord.append(cb+D/2)\n",
    "        ycoord.append(cb+D/2)\n",
    "        if sx!=0:\n",
    "            for i in range(nx-1):\n",
    "                xcoord.append(xcoord[-1]+sx)\n",
    "        if sy!=0:\n",
    "            for i in range(ny-1):\n",
    "                ycoord.append(ycoord[-1]+sy)\n",
    "        reinfarray_local = list(itertools.product(xcoord,ycoord))\n",
    "        \n",
    "        if layout == \"p\":\n",
    "            x0 = cb+D/2\n",
    "            xn = xcoord[-1]\n",
    "            y0 = cb+D/2\n",
    "            yn = ycoord[-1]\n",
    "            reinfarray_local = [e for e in reinfarray_local if e[0]==x0 or e[0]==xn or e[1]==y0 or e[1]==yn]\n",
    "                \n",
    "        # translate local origin to global\n",
    "        reinfarray_global=[]\n",
    "        for i in range(len(reinfarray_local)):\n",
    "            x_global = reinfarray_local[i][0] + x \n",
    "            y_global = reinfarray_local[i][1] + y \n",
    "            reinfarray_global.append([A,x_global,y_global])\n",
    "            \n",
    "        return reinfarray_global\n",
    "\n",
    "\n",
    "\n",
    "    def draw_web_bars(self,x,y,b,h,cb,s_target,A,D,ncurtain,direction):\n",
    "        \"\"\"\n",
    "        This function generates the x,y coordinate of boundary element steel based on\n",
    "        the following input:\n",
    "        x : dx from origin to lower left corner of web\n",
    "        y : dy from origin to lower left corner of web\n",
    "        b : dimension along x\n",
    "        h : dimension along y\n",
    "        cb : clear cover to face of rebar\n",
    "        s_target : target spacing of bars\n",
    "        A : area of rebar\n",
    "        D : diameter of rebar\n",
    "        ncurtain: number of curtains\n",
    "        direction: vertical or horizontal\n",
    "        \"\"\"\n",
    "        xcoord=[]\n",
    "        ycoord=[]\n",
    "        reinfarray_local=[]\n",
    "        if direction == \"v\":\n",
    "            if ncurtain==1:\n",
    "                N_L = math.floor((h)/s_target)\n",
    "                if N_L==0:\n",
    "                    return []\n",
    "                s_L = (h)/(N_L)\n",
    "                xcoord.append(0)\n",
    "                ycoord.append(s_L/2)\n",
    "                for i in range(N_L-1):\n",
    "                    ycoord.append(ycoord[-1]+s_L)\n",
    "                reinfarray_local = list(itertools.product(xcoord,ycoord))\n",
    "            else:\n",
    "                s_t = (b-cb-cb-D)/(ncurtain-1)\n",
    "                N_L = math.floor((h)/s_target)\n",
    "                if N_L==0:\n",
    "                    return []\n",
    "                s_L = (h)/(N_L)\n",
    "                xcoord.append(cb+D/2)\n",
    "                ycoord.append(s_L/2)\n",
    "                for i in range(ncurtain-1):\n",
    "                    xcoord.append(xcoord[-1]+s_t)\n",
    "                for i in range(N_L-1):\n",
    "                    ycoord.append(ycoord[-1]+s_L)\n",
    "                reinfarray_local = list(itertools.product(xcoord,ycoord))\n",
    "        elif direction == \"h\":\n",
    "            if ncurtain==1:\n",
    "                N_L = math.floor((b)/s_target)\n",
    "                if N_L==0:\n",
    "                    return []\n",
    "                s_L = (b) / (N_L)\n",
    "                ycoord.append(0)\n",
    "                xcoord.append(s_L/2)\n",
    "                for i in range(N_L-1):\n",
    "                    xcoord.append(xcoord[-1]+s_L)\n",
    "                reinfarray_local = list(itertools.product(xcoord,ycoord))\n",
    "            else:\n",
    "                s_t = (h-cb-cb-D)/(ncurtain-1)\n",
    "                N_L = math.floor((b)/s_target)\n",
    "                if N_L==0:\n",
    "                    return []\n",
    "                s_L = (b) / (N_L)\n",
    "                ycoord.append(cb+D/2)\n",
    "                xcoord.append(s_L/2)\n",
    "                for i in range(ncurtain-1):\n",
    "                    ycoord.append(ycoord[-1]+s_t)\n",
    "                for i in range(N_L-1):\n",
    "                    xcoord.append(xcoord[-1]+s_L)\n",
    "                reinfarray_local = list(itertools.product(xcoord,ycoord))\n",
    "        # translate local origin to global\n",
    "        reinfarray_global=[]\n",
    "        for i in range(len(reinfarray_local)):\n",
    "            x_global = reinfarray_local[i][0] + x \n",
    "            y_global = reinfarray_local[i][1] + y \n",
    "            reinfarray_global.append([A,x_global,y_global])\n",
    "        return reinfarray_global\n",
    "\n",
    "\n",
    "\n",
    "    def get_reinf_coord(self):\n",
    "        \"\"\"\n",
    "        function to generate reinforcement coordinates\n",
    "        \"\"\"\n",
    "        # Draw BE steel\n",
    "        BE_reinf=[]\n",
    "        dx = -self.b/2\n",
    "        dy = -self.h/2\n",
    "        A = REBAR[self.BE_bar][1]\n",
    "        D = REBAR[self.BE_bar][0]\n",
    "        BE_reinf.append(self.draw_BE_bars(dx,dy,self.b,self.Lbe,self.cb,self.nx,self.ny,A,D,\"p\"))\n",
    "        dx = -self.b/2\n",
    "        dy = self.h/2 - self.Lbe\n",
    "        A = REBAR[self.BE_bar][1]\n",
    "        D = REBAR[self.BE_bar][0]\n",
    "        BE_reinf.append(self.draw_BE_bars(dx,dy,self.b,self.Lbe,self.cb,self.nx,self.ny,A,D,\"p\"))\n",
    "        BE_reinf = BE_reinf[0] + BE_reinf[1]\n",
    "        \n",
    "        \n",
    "        # Draw web steel\n",
    "        dx = -self.b/2\n",
    "        dy = -self.h/2 + self.Lbe\n",
    "        h = self.h - 2*self.Lbe\n",
    "        s = self.s_target\n",
    "        A = REBAR[self.web_bar][1]\n",
    "        D = REBAR[self.web_bar][0]\n",
    "        web_reinf=self.draw_web_bars(dx,dy,self.b,h,self.cb,s,A,D,self.n_curtain,'v')\n",
    "\n",
    "        # combine all reinf coordinate into one list\n",
    "        reinfarray = BE_reinf + web_reinf\n",
    "                \n",
    "        return BE_reinf,web_reinf,reinfarray\n",
    "\n",
    "\n",
    "        \n",
    "    def PM(self,c):\n",
    "        \"\"\"\n",
    "        Algorithm to determine PM surface as follows (From CRSI)\n",
    "            0.) determine Pmax\n",
    "            1.) assume c values from 0.01 to N where the last step is sufficiently close to Pmax\n",
    "            2.) determine beta\n",
    "            3.) determine a\n",
    "            4.) calculate compression force resultant C\n",
    "            5.) loop through every single reinforcement\n",
    "                5a.) calculate rebar depth\n",
    "                5b.) calculate rebar strain & stress\n",
    "                5c.) calculate rebar force\n",
    "            6.) calculate Pn and Mn of the section\n",
    "            7.) back to step 1, assume another c value\n",
    "        \"\"\"\n",
    "        # step 2\n",
    "        if self.fpc < 4:\n",
    "            beta = 0.85\n",
    "        elif self.fpc > 8:\n",
    "            beta = 0.65\n",
    "        else:\n",
    "            beta = 0.85 - 0.05*(self.fpc*1000-4000)/1000\n",
    "        \n",
    "        # step 3\n",
    "        a = beta*c\n",
    "        \n",
    "        # step 4\n",
    "        C = 0.85*self.fpc*a*self.b\n",
    "        \n",
    "        # step 5 (-ve is tensile strain)\n",
    "        F_list=[]\n",
    "        d_list=[]\n",
    "        for bar in self.reinf_coord:\n",
    "            # step 5a\n",
    "            di = self.h/2 - bar[2]\n",
    "            Asi = bar[0]\n",
    "            \n",
    "            # step 5b\n",
    "            esi = 0.003*(c-di)/c\n",
    "            fsi = 29000*esi\n",
    "            if fsi<0 and fsi<-self.fy:\n",
    "                fsi = -self.fy\n",
    "            elif fsi>0 and fsi>self.fy:\n",
    "                fsi = self.fy\n",
    "                \n",
    "            # step 5c\n",
    "            if di > a:\n",
    "                Fsi = fsi*Asi\n",
    "            else:\n",
    "                Fsi = (fsi-0.85*self.fpc)*Asi\n",
    "            F_list.append(Fsi)\n",
    "            d_list.append(di)\n",
    "            \n",
    "        # step 6\n",
    "        P = C + sum(F_list)\n",
    "        M = 0.5*C*(self.h-a) + sum([A*(0.5*self.h-B) for A,B in zip(F_list,d_list)])\n",
    "        M = M/12\n",
    "        \n",
    "        # step 7 calculuate strain at extreme fiber and phi factor\n",
    "        ey = self.fy / 29000\n",
    "        d = self.h - self.cb\n",
    "        et = (d-c)/c * 0.0031\n",
    "        \n",
    "        if et >ey+0.003:\n",
    "            phi = 0.9\n",
    "        elif et>ey and et<ey+0.003:\n",
    "            phi = 0.75 + 0.15*(et-ey)/((ey+0.003)-ey)\n",
    "        elif et<ey:\n",
    "            phi = 0.65\n",
    "        \n",
    "        return P,M,c,phi\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "    def approx_PM(self,Pu,fy,lw,tw,fpc,lwb,lww,Asl1,Asl2):   \n",
    "        \"\"\"\n",
    "        determine interaction surface with approximate equation\n",
    "        Notation:\n",
    "            Mn1 = moment contribution from BE\n",
    "            Mn2 = moment contribution from web\n",
    "            Fy = yield strength of rebar\n",
    "            Pu = axial demand\n",
    "            Aslb = area of steel in boundary element\n",
    "            Aslw = area of steel in web\n",
    "            Asl1 = area of equivalent steel plate\n",
    "            Asl2 = area of boundary steel adjusted\n",
    "            lw = length of wall\n",
    "            lwb = length of BE\n",
    "            lww = length of wall web\n",
    "            tw = thickness of wall\n",
    "            c = depth of neutral axis\n",
    "            w = reinforcement ratio\n",
    "            alpha = axial load ratio\n",
    "            beta = stress block parameter\n",
    "            \n",
    "        1.) axial load ratio\n",
    "                alpha = Pu/fpc/lw/tw\n",
    "        2.) beta stress block\n",
    "                beta = 0.85 - 0.05*(fpc-4000)/1000, between 0.65 and 0.85\n",
    "        3.) reinforcement ratio\n",
    "                w = Asl1*Fy/lw/tw/fpc\n",
    "        4.) neutral axis depth\n",
    "                c = (w + alpha) / (2*w+0.85*beta)\n",
    "        5.) moment from boundary steel\n",
    "                Mn2 = Asl2*Fy*(lw-lwb)\n",
    "        6.) moment from web steel\n",
    "                Mn1 = 0.5*Asl1*Fy*lw*(1+Pu/Asl1/Fy)*(1-c/lw)\n",
    "        7.) total nominal moment capacity\n",
    "                Mn = Mn1 + Mn2\n",
    "        \"\"\"\n",
    "        ZETA=self.zeta\n",
    "        # step 1\n",
    "        Ag = self.b * self.h\n",
    "        Abe = len(self.BE_reinf) * self.BE_reinf[0][0]\n",
    "        Aweb = len(self.web_reinf) * self.web_reinf[0][0]\n",
    "        Ast = Abe + Aweb\n",
    "        Pmax = 0.85*self.fpc *(Ag - Ast) + self.fy * Ast\n",
    "        \n",
    "        alpha = Pu/(self.fpc*Ag)\n",
    "        \n",
    "        # step 2\n",
    "        if fpc < 4:\n",
    "            beta = 0.85\n",
    "        elif fpc > 8:\n",
    "            beta = 0.65\n",
    "        else:\n",
    "            beta = 0.85 - 0.05*(fpc*1000-4000)/1000\n",
    "            \n",
    "        # step 3\n",
    "        w = Asl1*fy/lw/tw/fpc\n",
    "        \n",
    "        # step 4\n",
    "        c = (w + alpha) / (2*w+0.85*beta)*lw\n",
    "        \n",
    "        # step 5\n",
    "        Mn2 = Asl2*fy*(lw-lwb)\n",
    "        \n",
    "        # step 6\n",
    "        Mn1 = 0.5*Asl1*fy*lw*(1+Pu/Asl1/fy)*(1-ZETA*c/lw)\n",
    "        \n",
    "        # step 7\n",
    "        Mn = (Mn1 + Mn2)/12\n",
    "        Pn = Pu\n",
    "        \n",
    "        # step 8 calculuate strain at extreme fiber and phi factor\n",
    "        ey = fy / 29000\n",
    "        d = self.h - self.cb\n",
    "        et = (d-c)/c * 0.003\n",
    "        if c<0:\n",
    "            et = ey+0.0031\n",
    "        \n",
    "        if et >ey+0.003:\n",
    "            phi = 0.9\n",
    "        elif et>ey and et<ey+0.003:\n",
    "            phi = 0.75 + 0.15*(et-ey)/(ey+0.003-ey)\n",
    "        elif et<ey:\n",
    "            phi = 0.65\n",
    "            \n",
    "        return Pn, Mn, c, phi\n",
    "            \n",
    "        \n",
    "\n",
    "    def determine_approx_PM(self):\n",
    "        \"\"\"\n",
    "        return a list of coordinates for the approximate PM surface\n",
    "        \"\"\"\n",
    "        # constants\n",
    "        fy = self.fy\n",
    "        lw = self.h\n",
    "        tw = self.b\n",
    "        fpc = self.fpc\n",
    "        lwb = self.Lbe\n",
    "        lww = self.h - 2*self.Lbe\n",
    "        Ag = lw*tw\n",
    "        \n",
    "        # determine steel quantity\n",
    "        Aslb = sum([A[0] for A in self.BE_reinf])/2\n",
    "        Aslw = sum([A[0] for A in self.web_reinf])\n",
    "        Astotal = Aslb*2 + Aslw\n",
    "        Asl1 = Aslw * lw/lww\n",
    "        Asl2 = Aslb - Aslw * lwb / lww\n",
    "              \n",
    "        # generate PM points\n",
    "        P_coord=[]\n",
    "        M_coord=[]\n",
    "        c_coord=[]\n",
    "        phi_coord=[]\n",
    "        P_list = np.linspace(self.Tmin,self.Pmax,100)\n",
    "        for p in P_list:\n",
    "            Pn,Mn,c,phi = self.approx_PM(p,fy,lw,tw,fpc,lwb,lww,Asl1,Asl2)\n",
    "            P_coord.append(Pn)\n",
    "            M_coord.append(Mn)\n",
    "            c_coord.append(c)\n",
    "            phi_coord.append(phi)\n",
    "        \n",
    "        self.PMsurface_approx = [P_coord,M_coord,c_coord,phi_coord]\n",
    "\n",
    "\n",
    "    def determine_PM(self):\n",
    "        \"\"\"\n",
    "        function used to determine accurate PM surface via strain compatibility\n",
    "        \"\"\"\n",
    "        # determine steel and concrete areas\n",
    "        Ag = self.b * self.h\n",
    "        Abe = len(self.BE_reinf) * self.BE_reinf[0][0]\n",
    "        Aweb = len(self.web_reinf) * self.web_reinf[0][0]\n",
    "        Ast = Abe + Aweb\n",
    "        self.Pmax = 0.85*self.fpc *(Ag - Ast) + self.fy * Ast\n",
    "        self.Tmin = -Ast * self.fy\n",
    "        \n",
    "        # first step with c = 0.01. Full tension\n",
    "        P_coord=[]\n",
    "        M_coord=[]\n",
    "        c_coord=[]\n",
    "        phi_coord=[]\n",
    "        cstep=0.01\n",
    "        P,M,c,phi = self.PM(cstep)\n",
    "        P_coord.append(P)\n",
    "        M_coord.append(M)\n",
    "        c_coord.append(c)\n",
    "        phi_coord.append(phi)\n",
    "        \n",
    "        \n",
    "        # loop and increment c by 1 inch every loop until reaching 0.8Po\n",
    "        P=0\n",
    "        cstep=1\n",
    "        while abs(P-self.Pmax)>300:\n",
    "            P,M,c,phi = self.PM(cstep)\n",
    "            P_coord.append(P)\n",
    "            M_coord.append(M)\n",
    "            c_coord.append(c)\n",
    "            phi_coord.append(phi)\n",
    "            cstep = cstep +1\n",
    "        \n",
    "        self.PMsurface = [P_coord,M_coord,c_coord,phi_coord]\n",
    "            \n",
    "    \n",
    "    def factored_PM(self):\n",
    "        self.PMsurface_factored = [[],[],[]]\n",
    "        self.PMsurface_approx_factored = [[],[],[]]\n",
    "        \n",
    "        for i in range(len(self.PMsurface[0])):\n",
    "            if self.PMsurface[0][i] < self.Pmax*0.8:\n",
    "                p=self.PMsurface[0][i] * self.PMsurface[3][i]\n",
    "                m=self.PMsurface[1][i] * self.PMsurface[3][i]\n",
    "                c=self.PMsurface[2][i]\n",
    "                self.PMsurface_factored[0].append(p)\n",
    "                self.PMsurface_factored[1].append(m)\n",
    "                self.PMsurface_factored[2].append(c)\n",
    "        \n",
    "        for i in range(len(self.PMsurface_approx[0])):\n",
    "            if self.PMsurface_approx[0][i] < self.Pmax*0.8:\n",
    "                p = self.PMsurface_approx[0][i]* self.PMsurface_approx[3][i]\n",
    "                m = self.PMsurface_approx[1][i]* self.PMsurface_approx[3][i]\n",
    "                c = self.PMsurface_approx[2][i]\n",
    "                self.PMsurface_approx_factored[0].append(p)\n",
    "                self.PMsurface_approx_factored[1].append(m)\n",
    "                self.PMsurface_approx_factored[2].append(c)\n",
    "        \n",
    "        # Add last point at 0.8Po\n",
    "        self.PMsurface_factored[0].append(self.PMsurface_factored[0][-1])\n",
    "        self.PMsurface_factored[1].append(0)\n",
    "        self.PMsurface_factored[2].append(self.PMsurface_factored[2][-1])\n",
    "        self.PMsurface_approx_factored[0].append(self.PMsurface_approx_factored[0][-1])\n",
    "        self.PMsurface_approx_factored[1].append(0)\n",
    "        self.PMsurface_approx_factored[2].append(self.PMsurface_approx_factored[2][-1])\n",
    "\n",
    "\n",
    "    def plot(self):\n",
    "        # initialize plot\n",
    "        fig, axs = plt.subplots(1,3,figsize=(16,9),gridspec_kw={'width_ratios':[1,3,3]})\n",
    "        \n",
    "        # plot section\n",
    "        axs[0].add_patch(patches.Rectangle([-self.b/2,-self.h/2], self.b, self.h, color = \"grey\" ))\n",
    "        BE_x, BE_y, web_x, web_y=[],[],[],[]\n",
    "        for coord in self.BE_reinf:\n",
    "            BE_x.append(coord[1])\n",
    "            BE_y.append(coord[2])\n",
    "        for coord in self.web_reinf:\n",
    "            web_x.append(coord[1])\n",
    "            web_y.append(coord[2])\n",
    "        #size of marker range from 2 to 6 for #4 to #11\n",
    "        websize =0.5*self.web_bar + 0.5\n",
    "        BEsize = 0.5*self.BE_bar + 0.5\n",
    "        axs[0].plot(BE_x,BE_y,marker=\".\",markersize=BEsize,color=\"black\",linestyle=\"None\")\n",
    "        axs[0].plot(web_x,web_y,marker=\".\",markersize=websize,color=\"black\",linestyle=\"None\")\n",
    "        # update styling \n",
    "        axs[0].set_xlim([-self.h/2*1.05,self.h/2*1.05])\n",
    "        axs[0].set_ylim([-self.h/2*1.05,self.h/2*1.05])\n",
    "        axs[0].set_title(\"Section Layout - h = {:.1f} ft\".format(self.h))\n",
    "        \n",
    "        \n",
    "        # plot PM\n",
    "        axs[1].plot(self.PMsurface_approx[1],self.PMsurface_approx[0],\n",
    "                    color=\"deepskyblue\",linestyle=\"dashed\",label=\"Approximation\")\n",
    "        axs[1].plot(self.PMsurface[1],self.PMsurface[0],color=\"darkviolet\",label=\"Strain Compatibility\")\n",
    "        # axis limits\n",
    "        Mmax=max(self.PMsurface[1])\n",
    "        Pmax=max(self.PMsurface[0])\n",
    "        Pmin=min(self.PMsurface[0])\n",
    "        axs[1].set_xlim([0,Mmax*1.05])\n",
    "        #axs[1].set_ylim([Pmin*1.05,Pmax*1.05])\n",
    "        #axs[1].set_xlim([-200000,200000])\n",
    "        #axs[1].set_ylim([-5000,50000])\n",
    "        # update styling\n",
    "        axs[1].set_title(\"PM Surface Comparison\")\n",
    "        axs[1].xaxis.grid()\n",
    "        axs[1].yaxis.grid()\n",
    "        axs[1].axhline(0,color=\"black\")\n",
    "        axs[1].axvline(0,color=\"black\")\n",
    "        axs[1].set_ylabel(\"Axial (kips)\")\n",
    "        axs[1].set_xlabel(\"Moment (kips.ft)\")\n",
    "        axs[1].legend(loc=\"upper right\")\n",
    "        \n",
    "        \n",
    "        \n",
    "        # plot P vs. c\n",
    "        axs[2].plot(self.PMsurface_approx[2],self.PMsurface_approx[0],color=\"deepskyblue\",linestyle=\"dashed\",label=\"Approximation\")\n",
    "        axs[2].plot(self.PMsurface[2],self.PMsurface[0],color=\"darkviolet\",label=\"Strain Compatibility\")\n",
    "        # update styling\n",
    "        axs[2].set_title(\"P vs. c\")\n",
    "        axs[2].xaxis.grid()\n",
    "        axs[2].yaxis.grid()\n",
    "        axs[2].set_xlabel(\"Neutral Axis Depth (in)\")\n",
    "        axs[2].set_ylabel(\"Axial Load (kips)\")\n",
    "        axs[2].legend(loc=\"upper right\")\n",
    "        # axis limits\n",
    "        Pmin=min(self.PMsurface[0])\n",
    "        Pmax=max(self.PMsurface[0])\n",
    "        cmax=max(self.PMsurface[2])\n",
    "        axs[2].set_ylim([Pmin*1.05,Pmax*1.05])\n",
    "        axs[2].set_xlim([0,cmax*1.05])\n",
    "        axs[2].axhline(0,color=\"black\")\n",
    "        axs[2].axvline(0,color=\"black\")\n",
    "        \n",
    "        return fig\n",
    "\n",
    "    def plot_factored(self):\n",
    "        # initialize plot\n",
    "        fig, axs = plt.subplots(1,3,figsize=(16,9),gridspec_kw={'width_ratios':[1,3,3]})\n",
    "        \n",
    "        # plot section\n",
    "        axs[0].add_patch(patches.Rectangle([-self.b/2,-self.h/2], self.b, self.h, color = \"grey\" ))\n",
    "        BE_x, BE_y, web_x, web_y=[],[],[],[]\n",
    "        for coord in self.BE_reinf:\n",
    "            BE_x.append(coord[1])\n",
    "            BE_y.append(coord[2])\n",
    "        for coord in self.web_reinf:\n",
    "            web_x.append(coord[1])\n",
    "            web_y.append(coord[2])\n",
    "        #size of marker range from 2 to 6 for #4 to #11\n",
    "        websize =0.5*self.web_bar + 0.5\n",
    "        BEsize = 0.5*self.BE_bar + 0.5\n",
    "        axs[0].plot(BE_x,BE_y,marker=\".\",markersize=BEsize,color=\"black\",linestyle=\"None\")\n",
    "        axs[0].plot(web_x,web_y,marker=\".\",markersize=websize,color=\"black\",linestyle=\"None\")\n",
    "        # update styling \n",
    "        axs[0].set_xlim([-self.h/2*1.05,self.h/2*1.05])\n",
    "        axs[0].set_ylim([-self.h/2*1.05,self.h/2*1.05])\n",
    "        axs[0].set_title(\"Section Layout - h = {:.1f} ft\".format(self.h))\n",
    "        \n",
    "        \n",
    "        # plot PM\n",
    "        axs[1].plot(self.PMsurface_approx_factored[1],self.PMsurface_approx_factored[0],\n",
    "                    color=\"deepskyblue\",linestyle=\"dashed\",label=\"Approximation\")\n",
    "        axs[1].plot(self.PMsurface_factored[1],self.PMsurface_factored[0],color=\"darkviolet\",label=\"Strain Compatibility\")\n",
    "        # axis limits\n",
    "        Mmax=max(self.PMsurface_factored[1])\n",
    "        Pmax=max(self.PMsurface_factored[0])\n",
    "        Pmin=min(self.PMsurface_factored[0])\n",
    "        axs[1].set_xlim([0,Mmax*1.05])\n",
    "        #axs[1].set_ylim([Pmin*1.05,Pmax*1.05])\n",
    "        #axs[1].set_xlim([-200000,200000])\n",
    "        #axs[1].set_ylim([-5000,50000])\n",
    "        # update styling\n",
    "        axs[1].set_title(\"PM Surface Comparison\")\n",
    "        axs[1].xaxis.grid()\n",
    "        axs[1].yaxis.grid()\n",
    "        axs[1].axhline(0,color=\"black\")\n",
    "        axs[1].axvline(0,color=\"black\")\n",
    "        axs[1].set_ylabel(\"Axial (kips)\")\n",
    "        axs[1].set_xlabel(\"Moment (kips.ft)\")\n",
    "        axs[1].legend(loc=\"upper right\")\n",
    "        \n",
    "        \n",
    "        \n",
    "        # plot P vs. c\n",
    "        axs[2].plot(self.PMsurface_approx[2],self.PMsurface_approx[0],color=\"deepskyblue\",linestyle=\"dashed\",label=\"Approximation\")\n",
    "        axs[2].plot(self.PMsurface[2],self.PMsurface[0],color=\"darkviolet\",label=\"Strain Compatibility\")\n",
    "        # update styling\n",
    "        axs[2].set_title(\"P vs. c\")\n",
    "        axs[2].xaxis.grid()\n",
    "        axs[2].yaxis.grid()\n",
    "        axs[2].set_xlabel(\"Neutral Axis Depth (in)\")\n",
    "        axs[2].set_ylabel(\"Axial Load (kips)\")\n",
    "        axs[2].legend(loc=\"upper right\")\n",
    "        # axis limits\n",
    "        Pmin=min(self.PMsurface[0])\n",
    "        Pmax=max(self.PMsurface[0])\n",
    "        cmax=max(self.PMsurface[2])\n",
    "        axs[2].set_ylim([Pmin*1.05,Pmax*1.05])\n",
    "        axs[2].set_xlim([0,cmax*1.05])\n",
    "        axs[2].axhline(0,color=\"black\")\n",
    "        axs[2].axvline(0,color=\"black\")\n",
    "        \n",
    "        return fig"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Interactive Plot\n",
    "The following parameters may be changed by user:\n",
    "* $f'_c$ = 4 ksi to 8 ksi\n",
    "* $f_y$ = 60 ksi to 100 ksi\n",
    "* $t$ = 12\" to 36\"\n",
    "* $L$ = 60\" to 500\"\n",
    "* $L_{BE}$ = 12\" to 60\"\n",
    "* $A_{wbar}$ = \\#3 to \\#11\n",
    "* $A_{bbar}$ = \\#3 to \\#11\n",
    "* $s_{web}$ = 4\" to 12\"\n",
    "\n",
    "The following parameters were set to a constant value as they do not affect the results much:\n",
    "* clear cover will always be 2\"\n",
    "* boundary element will always be 3 layers\n",
    "* boundary element vertical bar spacing will always be 4\". Amount of steel changed by $L_{BE}$. Longer length equals more boundary steel\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [],
   "source": [
    "# default values\n",
    "h=160\n",
    "fpc=6\n",
    "fy=60\n",
    "b=24\n",
    "BE_bar=10\n",
    "web_bar=6\n",
    "s_target=6\n",
    "Lbe=24"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {},
   "outputs": [],
   "source": [
    "def plot(fpc,fy,b,h,BE_bar,Lbe,web_bar,s_target,zeta,flag):\n",
    "    nx=3\n",
    "    n_curtain=2\n",
    "    cb=2\n",
    "    s_BE=4\n",
    "    ny=round(Lbe/s_BE)\n",
    "    my_section = Section(fpc,fy,b,h,cb,nx,ny,BE_bar,Lbe,web_bar,s_target,n_curtain,zeta)\n",
    "    if flag ==\"nominal\":\n",
    "        PM_fig = my_section.plot()\n",
    "    else:\n",
    "        PM_fig = my_section.plot_factored()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [],
   "source": [
    "slider1 = ipywidgets.IntSlider(value=6,min=4,max=8,step=1,description=\"f'c\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider2 = ipywidgets.IntSlider(value=60,min=60,max=100,step=20,description=\"fy\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider3 = ipywidgets.IntSlider(value=24,min=12,max=48,step=6,description=\"b\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider4 = ipywidgets.IntSlider(value=160,min=60,max=500,step=20,description=\"L\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider5 = ipywidgets.IntSlider(value=24,min=12,max=60,step=6,description=\"L_be\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider6 = ipywidgets.IntSlider(value=6,min=4,max=12,step=2,description=\"spacing\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider7 = ipywidgets.IntSlider(value=6,min=3,max=11,step=1,description=\"web bar\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider8 = ipywidgets.IntSlider(value=10,min=3,max=11,step=1,description=\"boundary bar\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "slider9 = ipywidgets.FloatSlider(value=0.83,min=0.6,max=1.5,step=0.01,description=\"zeta\",\n",
    "                               disabled=False,continuous_update=True, readout=True)\n",
    "dropdown10 = ipywidgets.Dropdown(options=['factored','nominal'],value='nominal',description='Type',disabled=False)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "b97d63df82284dd099c1f5a5d036e564",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "interactive(children=(IntSlider(value=6, description=\"f'c\", max=8, min=4), IntSlider(value=60, description='fy…"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/plain": [
       "<function __main__.plot(fpc, fy, b, h, BE_bar, Lbe, web_bar, s_target, zeta, flag)>"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ipywidgets.interact(plot,fpc=slider1,fy=slider2,b=slider3,h=slider4,\n",
    "                    BE_bar=slider8,Lbe=slider5,web_bar=slider7,s_target=slider6,zeta=slider9,flag=dropdown10)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.9"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
